<?php

function parse($str)
{
    static $parser;
    if (!isset($parser)) {
        $parser = new Parser;
    }
    return $parser->parse($str);
}

class Parser
{
    var $fmt, $depth;

    function Parser($fmt = 'Formatter')
    {
        $this->fmt = new $fmt;
    }
    
    function parse($text)
    {
        global $script_name;
        $parsed = "";
        $text = htmlspecialchars($text);
        $text = preg_replace("/{{{(.+?)}}}/e", "'<code>'.\$this->_escape('$1').'</code>'", $text);
        $text = preg_replace("/{{{(.+?)}}}/se", "'<pre class=\"escape\">'.\$this->_escape('$1').'</pre>'", $text);
        $lines = explode("\n", $text);
        $this->depth = 0;
        foreach ($lines as $line) {
            if (trim($line) == "") {
                $parsed .= "</p><p>";
            }
            $line = preg_replace_callback("/##(.+?)##/", array(&$this, "_macro"), $line);
            $line = preg_replace("/(?<!\[\[)(([A-Z][a-z]+){2,})/e", "\$this->fmt->link_to('$0')", $line);
            $line = preg_replace("/\[\[(.+?)]]/e", "\$this->page_link('$1')", $line);
            $line = preg_replace("/'''(.+?)'''/e", "\$this->fmt->strong('$1')", $line);
            $line = preg_replace("/''(.+?)''/e", "\$this->fmt->em('$1')", $line);
            $line = preg_replace("/^-{4,}/", "</p><hr /><p>", $line);
            $line = preg_replace("/--(.+?)--/e", "\$this->fmt->del('$1')", $line);
            $line = preg_replace("/__(.+?)__/e", "\$this->fmt->ins('$1')", $line);
            $line = preg_replace("/^=== (.+)/e", "\$this->fmt->head(5,'$1')", $line);
            $line = preg_replace("/^== (.+)/e", "\$this->fmt->head(4,'$1')", $line);
            $line = preg_replace("/^= (.+)/e", "\$this->fmt->head(3,'$1')", $line);
            $line = preg_replace("/^(\s*)(.+)$/e", "\$this->_indent(strlen('$1')).(substr('$2',0,1)=='*'?\$this->fmt->list_item(substr('$2',1)):'$2')", $line);
            $line = preg_replace("/\\\\(.)/", "$1", $line);
            $parsed .= $line;
        }
        return "<p>".$parsed."</p>";
    }

    function page_link($str)
    {
        @list($name, $title) = explode('|', $str, 2);
        return $this->fmt->link_to($name, "", $title);
    }
    
    function _indent($depth)
    {
        if ($this->depth > $depth)
            $str = str_repeat($this->fmt->list_(0), $this->depth - $depth);
        else if ($this->depth < $depth)
            $str = str_repeat($this->fmt->list_(1), $depth - $this->depth);
        else
            $str = "";
        $this->depth = $depth;
        return $str;
    }
    
    function _escape($str)
    {
	    return preg_replace("/./", "\\\\$0", $str);
    }
    
    function _macro($m)
    {
        @list($a, $b) = explode(':', $m[1], 2);
        $args = explode(',', $b);
        $macro = "macro_".$a;
        if (function_exists($macro)) {
            if ($args)
                return call_user_func_array($macro, $args);
            else
                return call_user_func($macro);
        }
        return $m[0];
    }

    function isWikiName($page)
    {
        return preg_match("/([A-Z][a-z]+){2,}/", $page);
    }
}

?>
